#!/usr/bin/perl -w
#
# cleanup.cgi
# John Simpson <jms1@jms1.net> 2012-07-21
#
# Clean up old database records
#
# $Id: cleanup.cgi 123 2012-07-23 02:24:36Z jms1 $
#
###############################################################################
#
# Copyright (C) 2012 John M. Simpson.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or version 3 of the
# license, at your option.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

require 5.003 ;
use strict ;
use CGI qw ( :standard ) ;

use Rules ;

###############################################################################
###############################################################################
###############################################################################

require_login() ;

########################################
# Check for SA authorization

unless ( $access > 2 )
{
	logit ( "ERROR: attempt to run cleanup without SA authorization."
		. " access=[$access] userid=[$userid]"
		. " remote_ip=[$remote_ip]\n" ) ;
	redirect_home() ;
}

########################################
# If we're cancelling, cancel.

if ( param ( 'cancel' ) )
{
	logit "cancelled\n" ;
	redirect_home() ;
}

########################################
# Read form data.

my $clean_sessions = ( param ( 's'  ) || '' ) ;
my $clean_greylist = ( param ( 'g'  ) || '' ) ;
my $go             = ( param ( 'go' ) || '' ) ;

########################################
# If we're confirming, do the deed.

if ( $go )
{
	########################################
	# If we're cleaning sessions, do it.

	if ( $clean_sessions eq '1' )
	{
		my $now = time() ;
		my $query = "DELETE FROM sessions"
			. " WHERE ended OR expires < $now" ;
		SQL_do ( $query ) ;
	}

	########################################
	# If we're cleaning greylist entries, do it.

	if ( $clean_greylist eq '1' )
	{
		my $days = sprintf ( '%d' , ( param ( 'days' ) || '0' ) ) ;
		if ( $days > 0 )
		{
			my $limit = time() - 86400 * $days ;
			my $query = "DELETE FROM greylist WHERE last < $limit" ;
			SQL_do ( $query ) ;
		}
	}

	########################################
	# done

	redirect_home() ;
}

########################################
# Not doing anything yet. Count how many expired sessions exist.

my $now = time() ;
my $query = "SELECT COUNT(*) FROM sessions WHERE ended OR expires < $now" ;
my ( $scount ) = SQL_select ( $query ) ;

########################################
# Show the form.

print page_start ( "Database Cleanup" ) ;
print parse_template ( 'cleanup.txt' ,
	'scount' => $scount ) ;
print page_end() ;
exit 0 ;
