#!/usr/bin/perl -w
#
# edit.cgi
# John Simpson <jms1@jms1.net>
#
# $Id: edit.cgi 114 2012-07-22 01:22:46Z jms1 $
#
###############################################################################
#
# Copyright (C) 2012 John M. Simpson.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or version 3 of the
# license, at your option.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

require 5.003 ;
use strict ;
use CGI qw ( :standard ) ;

use Rules ;

###############################################################################
###############################################################################
###############################################################################

require_login() ;

########################################
# Check for cancel button.

if ( param ( 'cancel' ) )
{
	logit "cancelled\n" ;
	redirect_home() ;
}

########################################
# Read form data.

my $p_phase	= ( param ( 'phase'  ) || '' ) ;
my $p_seq	= ( param ( 'seq'    ) || '' ) ;
my $p_go	= ( param ( 'go'     ) || '' ) ;
my $p_value	= ( param ( 'value'  ) || '' ) ;
my $p_accept	= ( param ( 'accept' ) || 0  ) ? 1 : 0 ;
my $p_desc	= ( param ( 'desc'   ) || '' ) ;

########################################
# Basic sanity check

unless ( ( $p_phase =~ m|^\d+$| ) && ( $p_seq =~ m|^\d+| ) )
{
	logit "ERROR: invalid phase [$p_phase] or seq [$p_seq]\n" ;
	redirect_home() ;
}

########################################
# Make sure the user is authorized to work with rules in this phase.

my $query = "SELECT level FROM phases WHERE phase = $p_phase" ;
my $plevel = SQL_select ( $query ) ;
$plevel ||= 0 ;

if ( $access < $plevel )
{
	logit "ERROR: attempt to edit rule without proper access: phase=[$p_phase] seq=[$p_seq] plevel=[$plevel] access=[$access] userid=[$userid] focus[$focus]\n" ;
	redirect_home() ;
}

########################################
# Get the existing rule.

$query = 'SELECT id , recipient , type , sender , ip , rbl , delay , target'
	. ' , accept , description'
	. ' FROM rules'
	. ' WHERE ' . SQL_quote ( $focus ) . ' LIKE recipient'
	. " AND phase = $p_phase AND seq = $p_seq" ;
my @data = SQL_select ( $query ) ;

if ( $#data < 0 )
{
	logit "ERROR: rule not found: p_phase=[$p_phase] p_seq=[$p_seq] plevel=[$plevel] access=[$access] userid=[$userid] focus=[$focus]\n" ;
	redirect_home() ;
}

my $ruleid	= ( shift @data || -1 ) ;
my $r_recip	= ( shift @data || '' ) ;
my $r_type	= ( shift @data || '' ) ;
my $r_sender	= ( shift @data || '' ) ;
my $r_ip	= ( shift @data || '' ) ;
my $r_rbl	= ( shift @data || '' ) ;
my $r_delay	= ( shift @data || 0  ) ;
my $r_target	= ( shift @data || '' ) ;
my $r_accept	= ( ( shift @data || 0 ) ? 1 : 0 ) ;
my $r_desc	= ( shift @data || '' ) ;

if ( $r_recip eq '%' )
{
	$r_recip = 'All mail sent to the server' ;
}
elsif ( $r_recip =~ s|^%\@|| )
{
	$r_recip = "All mail sent to the domain $r_recip" ;
}

$r_ip =~ s|\s+|| ;

########################################
# Set variables for the rule type.

my $templ = '' ;
my $title = '' ;
my $value = '' ;
my $field = '' ;
my $a1sel = ( $r_accept ? 'checked' : '' ) ;
my $a0sel = ( $r_accept ? '' : 'checked' ) ;
my $validate = undef ;				# validation function
my $clear_count = 0 ;

if ( $r_type eq 'A' )
{
	$templ = 'edit-all.txt' ;
	$title = 'Editing all-messages rule' ;
	$value = '(All messages)' ;
	$field = '' ;
	$validate = undef ;
}
elsif ( $r_type eq 'D' )
{
	$templ = 'edit-debug.txt' ;
	$title = 'Editing debug rule' ;
	$value = '(All messages)' ;
	$field = '' ;
	$validate = undef ;
}
elsif ( $r_type eq 'E' )
{
	$templ = 'edit-email.txt' ;
	$title = 'Editing email rule' ;
	$value = $r_sender ;
	$field = 'sender' ;
	$validate = \&validate_regex ;
}
elsif ( $r_type eq 'G' )
{
	$templ = 'edit-grey.txt' ;
	$title = 'Editing greylist rule' ;
	$value = $r_delay ;
	$field = 'delay' ;
	$validate = \&validate_grey ;
}
elsif ( $r_type eq 'I' )
{
	$templ = 'edit-ip.txt' ;
	$title = 'Editing IP rule' ;
	$value = $r_ip ;
	$field = 'ip' ;
	$validate = \&validate_ip ;
}
elsif ( $r_type eq 'R' )
{
	$templ = 'edit-rbl.txt' ;
	$title = 'Editing rbl rule' ;
	$value = $r_rbl ;
	$field = 'rbl' ;
	$validate = \&validate_rbl ;
}
elsif ( $r_type eq 'T' )
{
	$templ = 'edit-target.txt' ;
	$title = 'Editing Target rule' ;
	$value = $r_target ;
	$field = 'target' ;
	$validate = \&validate_regex ;
}
elsif ( $r_type eq 'U' )
{
	$templ = 'edit-auth.txt' ;
	$title = 'Editing AUTH client rule' ;
	$value = '(Authenticated clients)' ;
	$field = '' ;
	$validate = undef ;
}
else
{
	fatal "Cannot edit rule type \"$r_type\"\n" ;
}

########################################
# If we're actually doing something, do it.

if ( $p_go )
{
	########################################
	# Trim whitespace from beginning and end, then validate the value.

	$p_value =~ s|^\s*|| ;
	$p_value =~ s|\s*$|| ;

	if ( $validate )
	{
		unless ( $validate->($p_value) )
		{
			bad ( "Invalid input: the \"$field\" field"
				. " cannot hold \"$p_value\"." ) ;
		}
	}

	########################################
	# Figure out what's changed.

	my @qf = () ;
	my @qd = () ;

	if ( $p_value ne $value )
	{
		my $qvalue = ( 'EIRT' =~ m|$r_type| )
			? SQL_quote ( $p_value )
			: $p_value ;

		if ( $field )
		{
			push ( @qf , $field  ) ;
			push ( @qd , $qvalue ) ;
		}

		$clear_count = 1 ;
	}

	if ( $p_accept != $r_accept )
	{
		push ( @qf , 'accept'  ) ;
		push ( @qd , ( $p_accept ? SQL_true() : SQL_false() ) ) ;

		$clear_count = 1 ;
	}

	if ( $p_desc ne $r_desc )
	{
		push ( @qf , 'description' ) ;
		push ( @qd , SQL_quote ( $p_desc ) ) ;
	}

	########################################
	# Build and run the query.

	if ( $#qf > -1 )
	{
		$query = 'UPDATE rules SET ' ;

		for my $n ( 0 .. $#qf )
		{
			$query .= ( $n ? ' , ' : '' )
				. "$qf[$n] = $qd[$n]" ;
		}

		if ( $clear_count )
		{
			$query .= ' , created = CURRENT_TIMESTAMP' ;
		}

		$query .= " WHERE id = $ruleid" ;

		SQL_do ( $query ) ;
	}

	########################################
	# outta here
	
	redirect_home() ;
}

########################################
# Not making changes. Show the form.

$r_desc =~ s|'|&apos;|g ;

print page_start ( $title ) ;
print parse_template ( $templ ,
	'phase'	=> $p_phase ,
	'seq'	=> $p_seq ,
	'recip'	=> $r_recip ,
	'value'	=> $value ,
	'a1sel'	=> ( $r_accept ? 'checked' : '' ) ,
	'a0sel'	=> ( $r_accept ? '' : 'checked' ) ,
	'desc'  => $r_desc ) ;
print page_end() ;
exit 0 ;
