#!/usr/bin/perl -w
#
# focus.cgi
# John Simpson <jms1@jms1.net>
#
# Change a web site user's focus.
#
# $Id: focus.cgi 106 2012-07-21 20:39:17Z jms1 $
#
###############################################################################
#
# Copyright (C) 2012 John M. Simpson.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or version 3 of the
# license, at your option.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

require 5.003 ;
use strict ;
use CGI qw ( :standard ) ;

use Rules ;

###############################################################################
###############################################################################
###############################################################################

require_login() ;

########################################
# Handle cancel button

if ( param ( 'cancel' ) )
{
	redirect_home() ;
}

########################################
# Read form data.

my $type = ( param ( 'type' ) || '' ) ;	# both forms
my $dom  = ( param ( 'dom'  ) || '' ) ;	# only on SA form
my $mbox = ( param ( 'mbox' ) || '' ) ;	# only on SA form
my $user = ( param ( 'user' ) || '' ) ;	# only on DA form

########################################
# If we have data, process it.

if ( $type eq 's' )
{
	set_focus ( "\@" ) ;
	redirect_home() ;
}

if ( $type eq 'd' )
{
	########################################
	# The DA form doesn't send a "dom" element.
	# If no "dom" element is present, use the domain from $userid.

	$dom ||= $userid ;
	$dom =~ s|^.*\@|| ;

	set_focus ( "\@$dom" ) ;
	redirect_home() ;
}

if ( ( $type eq 'm' ) && $mbox )
{
	$mbox =~ m|^(.+)\@(.+)| ;
	my ( $user , $dom ) = ( $1 , $2 ) ;
	$user =~ s|\-.*$|| ;

	if ( $user && $dom )
	{
		set_focus ( "$user\@$dom" ) ;
	}
	redirect_home() ;
}

if ( ( $type eq 'u' ) && $user )
{
	( $dom = $userid ) =~ s|^.*\@|| ;
	$user =~ s|\-.*$|| ;
	set_focus ( "$user\@$dom" ) ;
	redirect_home() ;
}

if ( $type eq 'x' )
{
	set_focus ( $userid ) ;
	redirect_home() ;
}

########################################
# If we have system admin rights, show that form.

if ( $access == 3 )
{
	my $ssel = '' ;
	my $dsel = '' ;
	my $dval = '' ;
	my $msel = '' ;
	my $mval = '' ;
	my $xsel = '' ;

	if ( $focus eq '@' )
	{
		$ssel = 'checked' ;
	}
	elsif ( $focus =~ m|^\@(.+)| )
	{
		$dsel = 'checked' ;
		$dval = $1 ;
	}
	elsif ( $focus && ( $focus ne $userid ) )
	{
		$msel = 'checked' ;
		$mval = $focus ;
	}
	else
	{
		$xsel = 'checked' ;
	}

	print page_start ( 'Change Focus' ) ;
	print parse_template ( 'focus-sa.txt',
		'ssel' => $ssel , 'dsel' => $dsel , 'dval' => $dval ,
		'msel' => $msel , 'mval' => $mval , 'xsel' => $xsel ,
		'focuslist' => get_focus_list() ) ;
	print page_end() ;
	exit 0 ;
}

########################################
# If we have domain admin rights, show that form.

if ( $access == 2 )
{
	my $dsel = '' ;
	my $usel = '' ;
	my $uval = '' ;
	my $xsel = '' ;

	( my $dom = $userid ) =~ s|^.*\@|| ;

	if ( $focus =~ m|^\@(.+)| )
	{
		$dsel = 'checked' ;
	}
	elsif ( $focus && ( $focus ne $userid ) )
	{
		$usel = 'checked' ;
		$uval = $focus ;
	}
	else
	{
		$xsel = 'checked' ;
	}


	print page_start ( 'Change Focus' ) ;
	print parse_template ( 'focus-da.txt' ,
		'dsel' => $dsel , 'dom'  => $dom ,
		'usel' => $usel , 'uval' => $uval , 'xsel' => $xsel ,
		'focuslist' => get_focus_list() ) ;
	print page_end() ;
	exit 0 ;
}

########################################
# If we don't have domain/system admin rights, focus cannot be changed.

set_focus ( $userid ) ;
redirect_home() ;
